import { fileURLToPath } from 'url';
import { useLogger, defineNuxtModule, requireModule, addTemplate, addPlugin, isNuxt2, addAutoImport, isNuxt3, addComponentsDir } from '@nuxt/kit';
import chalk from 'chalk';
import * as fse from 'fs-extra';
import { resolve, join } from 'pathe';
import { defu } from 'defu';
import { genImport } from 'knitwork';

const name = "@nuxtjs/sanity";
const version = "1.2.0";

const logger = useLogger("@nuxtjs/sanity");
function getDefaultSanityConfig(jsonPath) {
  try {
    const { projectId, dataset } = fse.readJSONSync(jsonPath).api;
    return { projectId, dataset };
  } catch {
    return {};
  }
}
const CONFIG_KEY = "sanity";
const module = defineNuxtModule({
  meta: {
    name,
    version,
    configKey: CONFIG_KEY,
    compatibility: {
      bridge: true
    }
  },
  defaults: (nuxt) => ({
    dataset: "production",
    apiVersion: "1",
    withCredentials: false,
    additionalClients: {},
    ...getDefaultSanityConfig(resolve(nuxt.options.rootDir, "./sanity.json"))
  }),
  async setup(options, nuxt) {
    if (!("useCdn" in options)) {
      options.useCdn = process.env.NODE_ENV === "production" && !options.token;
    }
    try {
      if (!options.minimal) {
        options.minimal = !requireModule("@sanity/client");
      }
    } catch {
      options.minimal = true;
      logger.info(`Enabling minimal client as ${chalk.bold("@sanity/client")} is not installed.`);
    }
    const { projectId, dataset } = nuxt.options.publicRuntimeConfig.sanity = defu(nuxt.options.publicRuntimeConfig.sanity, {
      useCdn: options.useCdn,
      projectId: options.projectId,
      dataset: options.dataset,
      apiVersion: options.apiVersion,
      withCredentials: options.withCredentials,
      token: options.token,
      additionalClients: options.additionalClients
    });
    if (!projectId) {
      logger.warn(`No Sanity project found. Make sure you specify a ${chalk.bold("projectId")} in your Sanity config.`);
    } else {
      logger.info(`Running with Sanity project ${chalk.bold(projectId)} (${chalk.bold(dataset)}).`);
    }
    const runtimeDir = fileURLToPath(new URL("./runtime", import.meta.url));
    nuxt.options.build.transpile.push(runtimeDir, "@nuxtjs/sanity");
    addTemplate({
      filename: "sanity-client.mjs",
      getContents: () => [
        options.minimal ? genImport(join(runtimeDir, "client"), ["createClient"]) : genImport("@sanity/client", "createClient"),
        "export { createClient }"
      ].join("\n")
    });
    if (options.globalHelper) {
      addPlugin({ src: join(runtimeDir, "plugin") });
      if (isNuxt2()) {
        nuxt.hook("prepare:types", ({ references }) => {
          references.push({ types: "@nuxtjs/sanity/dist/runtime/plugin" });
        });
      }
    }
    addAutoImport([
      { name: "createClient", as: "createSanityClient", from: "#build/sanity-client.mjs" },
      { name: "groq", as: "groq", from: join(runtimeDir, "groq") },
      { name: "useSanity", as: "useSanity", from: join(runtimeDir, "composables") },
      { name: "useLazySanityQuery", as: "useLazySanityQuery", from: join(runtimeDir, "composables") },
      isNuxt3() && { name: "useSanityQuery", as: "useSanityQuery", from: join(runtimeDir, "composables") }
    ].filter(Boolean));
    await addComponentsDir({
      path: join(runtimeDir, "components"),
      extensions: ["js", "ts", "mjs"]
    });
  }
});

export { module as default };
